from dataclasses import dataclass
from enum import Enum
from typing import Optional


class ClientException(Exception):
    def __init__(self, msg):
        self.msg = msg


class Cloud(Enum):
    AWS = "aws"


class JobStatus(Enum):
    PENDING = "pending"
    INITIALIZING = "initializing"
    RUNNING = "running"
    SHUTTING_DOWN = "shutting_down"
    FINISHED = "finished"
    CANCELLING = "cancelling"
    CANCELLED = "cancelled"
    ERROR = "error"
    UNKNOWN = "unknown"
    NOT_FOUND = "not found"
    UNAUTHORIZED = "unauthorized"
    UNREACHABLE = "unreachable"


class MachineStatus(Enum):
    PENDING = "pending"
    INITIALIZING = "initializing"
    RUNNING = "running"
    STOPPING = "stopping"
    STOPPED = "stopped"
    TERMINATING = "terminating"
    TERINATED = "terminated"
    ERROR = "error"
    UNKNOWN = "unknown"
    NOT_FOUND = "not found"
    UNAUTHORIZED = "unauthorized"
    UNREACHABLE = "unreachable"


@dataclass
class Job:
    id: str
    name: str
    status: JobStatus
    status_message: str
    setup_script: str
    run_script: str
    num_nodes: int
    env: dict
    resources: dict
    storage: dict
    cloud: Optional[Cloud]
    docker_image: Optional[str]
    created_timestamp: int
    updated_timestamp: int

    @classmethod
    def from_dict(cls, d):
        d["status"] = JobStatus(d["status"])
        if d.get("cloud", None):
            d["cloud"] = Cloud(d["cloud"])

        job = Job(**d)
        return job


@dataclass
class Machine:
    id: str
    name: str
    status: MachineStatus
    status_message: str
    setup_script: str
    env: dict
    resources: dict
    storage: dict
    cloud: Optional[Cloud]
    docker_image: Optional[str]

    @classmethod
    def from_dict(cls, d):
        d["status"] = MachineStatus(d["status"])
        if d.get("cloud", None):
            d["cloud"] = Cloud(d["cloud"])

        machine = Machine(**d)
        return machine



import json
import os
import shutil
import tempfile
import time
from typing import Dict, List, Optional, Union

import requests

from komo import printing
from komo.types import ClientException, Cloud, Job, JobStatus, Machine

KOMODO_API_URL = os.environ.get("KOMODO_API_URL", "https://api.komodoai.dev")

# refresh JWT token at this interval
TOKEN_REFRESH_INTERVAL = 60 * 60  # 1hr


class APIClient:
    def __init__(self):
        api_key_file = os.path.expanduser("~/.komo/api-key")
        if not os.path.isfile(api_key_file):
            raise ClientException(f"{api_key_file} does not exist")

        with open(api_key_file, "r") as f:
            api_key = f.read().strip()

        self.api_key = api_key
        self.refresh_token()

    @classmethod
    def register(cls, email: str, password: str):
        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
        }

        payload = json.dumps(
            {
                "email": email,
                "password": password,
            },
        )

        response = requests.request(
            "POST",
            f"{KOMODO_API_URL}/api/v1/auth/register",
            headers=headers,
            data=payload,
        )
        response.raise_for_status()

    def refresh_token(self):
        response = requests.request(
            "POST",
            f"{KOMODO_API_URL}/api/v1/auth/jwt/api_key/login?api_key={self.api_key}",
        )
        response.raise_for_status()
        auth = response.json()

        self._token = auth["token"]
        self.last_token_refresh_time = time.time()

    @property
    def token(self):
        curr_time = time.time()
        if (curr_time - self.last_token_refresh_time) > TOKEN_REFRESH_INTERVAL:
            self.refresh_token()

        return self._token

    @classmethod
    def get_api_key(cls, email: str, password: str):
        files = {
            "username": (None, email),
            "password": (None, password),
        }

        response = requests.request(
            "POST",
            f"{KOMODO_API_URL}/api/v1/auth/jwt/login",
            files=files,
        )
        response.raise_for_status()
        auth = response.json()

        token, token_type = auth["access_token"], auth["token_type"]

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "Authorization": f"{token_type} {token}",
        }

        response = requests.request(
            "GET",
            f"{KOMODO_API_URL}/api/v1/auth/jwt/api_key",
            headers=headers,
        )
        response.raise_for_status()
        auth = response.json()

        api_key = auth["api_key"]
        return api_key

    def api_request(
        self,
        method: str,
        url: str,
        files: Dict = None,
        data: Dict = None,
    ) -> Union[Dict, List]:  # Not using | for > 1 return type for < Py 3.10 compat
        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "Authorization": f"Bearer {self.token}",
        }
        # Generic way to make HTTP requests w/ standard error handling
        # TODO: debug-level info logs?
        response = requests.request(
            method,
            url,
            headers=headers,
            files=files,
            data=data,
        )
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            # Catch HTTP errors like 401, 500 etc.
            text = e.response.text
            try:
                j = e.response.json()
            except requests.exceptions.JSONDecodeError:
                j = None
            if j:
                if "detail" in j:
                    text = j["detail"]
            print(f"Got HTTP Error Code {e.response.status_code}: {text}")
            exit(1)

        retval = response.json()
        return retval

    def get_user_id(self):
        result = self.api_request(
            "GET",
            f"{KOMODO_API_URL}/api/v1/user-id",
        )

        return result["user-id"]

    def connect_aws(self, iam_role_arn):
        self.api_request(
            "POST",
            f"{KOMODO_API_URL}/api/v1/aws/connect?iam_role_arn={iam_role_arn}",
        )

    def create_job(
        self,
        setup_script: str,
        run_script: str,
        num_nodes: int,
        env: Dict[str, str],
        resources: dict,
        storage: dict,
        workdir: Optional[str] = None,
        cloud: Optional[Cloud] = None,
        docker_image: Optional[str] = None,
        name: Optional[str] = None,
    ) -> Job:
        payload = {
            "name": name,
            "setup_script": setup_script,
            "run_script": run_script,
            "num_nodes": num_nodes,
            "env": env,
            "resources": resources,
            "storage": storage,
            "cloud": cloud.value,
            "docker_image": docker_image,
        }

        if workdir:
            with tempfile.TemporaryDirectory() as td:
                workdir_zipfile = os.path.join(td, "workdir")
                workdir_zipfile = shutil.make_archive(
                    workdir_zipfile, "zip", workdir, workdir
                )

                upload_info = self.api_request(
                    "GET", f"{KOMODO_API_URL}/api/v1/jobs/upload_info"
                )
                workdir_upload_id = upload_info["upload_id"]
                upload_url = upload_info["url"]
                fields = upload_info["fields"]

                with open(workdir_zipfile, "rb") as f:
                    files = {"file": f}

                    if workdir == ".":
                        workdir_str = "current working directory"
                    else:
                        workdir_str = f"directory {workdir}"
                    printing.info(f"Uploading {workdir_str}")
                    response = requests.request(
                        "POST",
                        url=upload_url,
                        data=fields,
                        files=files,
                    )

                    response.raise_for_status()
                    printing.success(f"Uploaded {workdir_str}")

                payload["workdir_upload_id"] = workdir_upload_id

        payload = json.dumps(payload)
        job_dict = self.api_request(
            "POST", f"{KOMODO_API_URL}/api/v1/jobs", data=payload
        )
        job = Job.from_dict(job_dict)

        return job

    def get_job(self, job_id: str):
        job_dict = self.api_request("GET", f"{KOMODO_API_URL}/api/v1/jobs/{job_id}")
        job = Job.from_dict(job_dict)
        return job

    def get_jobs(self):
        job_dicts = self.api_request("GET", f"{KOMODO_API_URL}/api/v1/jobs")
        jobs = [Job.from_dict(d) for d in job_dicts]
        return jobs

    def _get_log_chunk(
        self,
        job_id: str,
        next_token: Optional[str] = None,
        start_time: Optional[int] = None,
        end_time: Optional[int] = None,
    ):
        url = f"{KOMODO_API_URL}/api/v1/jobs/{job_id}/logs"
        if next_token or start_time or end_time:
            url += "?"

        args = []
        if next_token:
            args.append(f"next_token={next_token}")
        if start_time:
            args.append(f"start_time={start_time * 1000}")
        if end_time:
            args.append(f"end_time={end_time * 1000}")

        url += "&".join(args)

        logs = self.api_request("GET", url)

        return logs

    def print_job_logs(self, job_id: str, follow: bool):
        job = self.get_job(job_id)
        if job.status in [JobStatus.PENDING, JobStatus.INITIALIZING]:
            raise Exception(f"Job {job_id} has not started")

        live = (
            job.status == JobStatus.RUNNING
            or job.status
            in [
                JobStatus.CANCELLED,
                JobStatus.CANCELLING,
                JobStatus.FINISHED,
                JobStatus.SHUTTING_DOWN,
            ]
            and (time.time() - job.updated_timestamp) < 30
        )

        # time between server queries, in seconds (so we don't overload the server)
        TIME_BETWEEN_QUERIES = 1
        # the number of seconds after a job finishes to wait before assuming no more logs are coming
        JOB_FINISH_WAIT_TIME = 30
        next_token = None
        job_finished_time = None
        last_query_time = time.time() - TIME_BETWEEN_QUERIES
        end_time = None
        if not follow:
            end_time = int(time.time())
        while True:
            time.sleep(max(TIME_BETWEEN_QUERIES - (time.time() - last_query_time), 0))
            response = self._get_log_chunk(job_id, next_token, end_time=end_time)
            last_query_time = time.time()

            for event in response["logs"]:
                printing.info(event["message"])

            if response["next_token"] == next_token:
                # we've reached the end of the currently available logs
                if not follow:
                    break

                job = self.get_job(job_id)
                if job.status != JobStatus.RUNNING:
                    if live:
                        if job_finished_time is None:
                            job_finished_time = time.time()
                        elif (time.time() - job_finished_time) >= JOB_FINISH_WAIT_TIME:
                            break

            next_token = response["next_token"]

    def cancel_job(self, job_id: str):
        self.api_request("POST", f"{KOMODO_API_URL}/api/v1/jobs/{job_id}/cancel")

    def get_private_ssh_key(self) -> str:
        response = self.api_request("GET", f"{KOMODO_API_URL}/api/v1/ssh-key")
        ssh_key = response["ssh-key"]
        return ssh_key

    def get_job_host_info(self, job_id: str):
        host_info = self.api_request(
            "GET", f"{KOMODO_API_URL}/api/v1/jobs/{job_id}/host-info"
        )
        return host_info

    def create_machine(
        self,
        setup_script: str,
        env: Dict[str, str],
        resources: dict,
        storage: dict,
        workdir: Optional[str] = None,
        cloud: Optional[Cloud] = None,
        docker_image: Optional[str] = None,
        name: Optional[str] = None,
    ) -> Machine:
        payload = {
            "name": name,
            "setup_script": setup_script,
            "env": env,
            "resources": resources,
            "storage": storage,
            "cloud": cloud.value,
            "docker_image": docker_image,
        }

        if workdir:
            with tempfile.TemporaryDirectory() as td:
                workdir_zipfile = os.path.join(td, "workdir")
                workdir_zipfile = shutil.make_archive(
                    workdir_zipfile, "zip", workdir, workdir
                )

                upload_info = self.api_request(
                    "GET", f"{KOMODO_API_URL}/api/v1/jobs/upload_info"
                )
                workdir_upload_id = upload_info["upload_id"]
                upload_url = upload_info["url"]
                fields = upload_info["fields"]

                with open(workdir_zipfile, "rb") as f:
                    files = {"file": f}

                    if workdir == ".":
                        workdir_str = "current working directory"
                    else:
                        workdir_str = f"directory {workdir}"
                    printing.info(f"Uploading {workdir_str}")
                    response = requests.request(
                        "POST",
                        url=upload_url,
                        data=fields,
                        files=files,
                    )

                    response.raise_for_status()
                    printing.success(f"Uploaded {workdir_str}")

                payload["workdir_upload_id"] = workdir_upload_id

        payload = json.dumps(payload)
        machine_dict = self.api_request(
            "POST", f"{KOMODO_API_URL}/api/v1/machines", data=payload
        )
        machine = Machine.from_dict(machine_dict)

        return machine

    def get_machines(self) -> List[Machine]:
        machine_dicts = self.api_request("GET", f"{KOMODO_API_URL}/api/v1/machines")
        machines = [Machine.from_dict(d) for d in machine_dicts]
        return machines

    def get_machine(self, machine_name: str) -> Machine:
        machine_dict = self.api_request(
            "GET", f"{KOMODO_API_URL}/api/v1/machines/{machine_name}"
        )
        machine = Machine.from_dict(machine_dict)
        return machine

    def terminate_machine(self, machine_name: str):
        self.api_request("DELETE", f"{KOMODO_API_URL}/api/v1/machines/{machine_name}")

    def get_machine_host_info(self, machine_name: str):
        host_info = self.api_request(
            "GET", f"{KOMODO_API_URL}/api/v1/machines/{machine_name}/host-info"
        )
        return host_info



import time
from typing import Optional

import click

from komo import printing
from komo.core import create_machine, get_machine
from komo.types import Cloud, MachineStatus


@click.command("create")
@click.option("--gpus", type=str, default=None)
@click.option("--cloud", "-c", type=str, default=None)
@click.option("--detach", "-d", is_flag=True, default=False)
@click.argument("name", nargs=1)
def cmd_create(
    gpus: Optional[str],
    cloud: Optional[str],
    detach: bool,
    name: str,
):
    if cloud:
        cloud = Cloud(cloud)
    machine = create_machine(gpus, name, cloud)
    printing.success(f"Machine {machine.name} successfully created")

    if detach:
        return

    printing.info("Waiting for machine to start...")

    last_messsage = None
    while True:
        machine = get_machine(name)

        should_break = False
        error = False
        if machine.status in [MachineStatus.PENDING, MachineStatus.INITIALIZING]:
            pass
        elif machine.status == MachineStatus.RUNNING:
            should_break = True
        else:
            should_break = True
            error = True

        if machine.status_message and machine.status_message != last_messsage:
            if error:
                printing.error(machine.status_message)
            else:
                printing.info(machine.status_message)

            last_messsage = machine.status_message

        if should_break:
            break

        time.sleep(5)

    if machine.status == MachineStatus.RUNNING:
        printing.success(f"Machine {name} successfully created")



import os

import yaml

from komo.constants import PROJECT_FILE_NAME
from komo.types import Cloud


def _set_default(config, key, value):
    if key not in config:
        config[key] = value


def _load_project_config():
    project_file = os.path.join(os.getcwd(), PROJECT_FILE_NAME)
    if os.path.isfile(project_file):
        with open(project_file, "r") as f:
            project_config = yaml.load(f, Loader=yaml.SafeLoader)
            if "cloud" in project_config:
                project_config["cloud"] = Cloud(project_config["cloud"])
    else:
        project_config = {
            # if a project config wasn't provided at all, we assume the cwd should
            # be copied to the workdir
            "workdir": "."
        }

    _set_default(project_config, "resources", {"cpus": "1+"})
    _set_default(project_config, "workdir", None)
    _set_default(project_config, "setup", "")
    _set_default(project_config, "storage", {})

    return project_config



import click

from komo import printing
from komo.core import cancel_job


@click.command("cancel")
@click.argument(
    "job_id",
    type=str,
)
def cmd_cancel(job_id: str):
    cancel_job(job_id)
    printing.success(f"Job {job_id} is being cancelled")



import os
import shutil
import subprocess
from typing import List, Optional

import yaml

from komo.api_client import APIClient
from komo.types import Cloud, Job, Machine, MachineStatus
from komo.utils import _load_project_config


def register(email: str, password: str):
    APIClient.register(email, password)
    login(email, password)


def login(email: str, password: str):
    komo_dir = os.path.expanduser("~/.komo")
    os.makedirs(komo_dir, exist_ok=True)
    api_key = APIClient.get_api_key(email, password)
    api_key_file = os.path.join(komo_dir, "api-key")
    with open(api_key_file, "w") as f:
        f.write(api_key)


def run_command(
    command: str,
    num_nodes: Optional[int] = None,
    gpus: Optional[str] = None,
    name: Optional[str] = None,
    cloud: Optional[Cloud] = None,
):
    """
    Run the given command.
    """
    project_config = _load_project_config()
    api_client = APIClient()

    resources = project_config.get("resources", {})
    if gpus:
        resources["gpus"] = gpus

    if not cloud:
        cloud = project_config.get("cloud", None)

    if num_nodes is None:
        num_nodes = 1

    if not cloud:
        raise Exception("Cloud not provided")

    job = api_client.create_job(
        project_config.get("setup", ""),
        command,
        num_nodes,
        project_config.get("env", {}),
        resources,
        project_config.get("storage", {}),
        project_config.get("workdir", "."),
        cloud=cloud,
        docker_image=project_config.get("docker_image", None),
        name=name,
    )

    return job


def _override_project_config(project_config: dict, job_config: dict, key: str):
    if job_config.get(key, None):
        if key in project_config:
            job_config[key] = project_config[key]


def run_job_file(
    job_file: str,
    num_nodes: Optional[int] = None,
    gpus: Optional[str] = None,
    name: Optional[str] = None,
    cloud: Optional[Cloud] = None,
):
    """
    Run the given job file
    """
    project_config = _load_project_config()
    api_client = APIClient()

    with open(job_file, "r") as f:
        job_config = yaml.safe_load(f)

    for key in [
        "setup",
        "run",
        "num_nodes",
        "env",
        "resources",
        "storage",
        "workdir",
        "cloud",
        "docker_image",
        "name",
    ]:
        _override_project_config(project_config, job_config, key)

    if num_nodes is not None:
        job_config["num_nodes"] = num_nodes

    if gpus is not None:
        job_config["resources"]["gpus"] = gpus

    if name is not None:
        job_config["name"] = name

    if cloud is not None:
        job_config["cloud"] = cloud

    if "run" not in job_config:
        raise Exception("run script not provided")

    if "cloud" not in job_config:
        raise Exception("Cloud not provided")

    job = api_client.create_job(
        job_config.get("setup", ""),
        job_config["run"],
        job_config.get("num_nodes", 1),
        job_config.get("env", {}),
        job_config.get("resources", {}),
        job_config.get("storage", {}),
        job_config.get("workdir", "."),
        job_config["cloud"],
    )

    return job


def list_jobs() -> List[Job]:
    api_client = APIClient()
    jobs = api_client.get_jobs()
    return jobs


def get_job(job_id) -> Job:
    api_client = APIClient()
    job = api_client.get_job(job_id)
    return job


def print_job_logs(job_id, follow: bool = False):
    api_client = APIClient()
    api_client.print_job_logs(job_id, follow)


def cancel_job(job_id):
    api_client = APIClient()
    api_client.cancel_job(job_id)


def _get_private_ssh_key() -> str:
    api_client = APIClient()
    ssh_key = api_client.get_private_ssh_key()
    return ssh_key


def ssh_job(job_id):
    api_client = APIClient()

    host_info = api_client.get_job_host_info(job_id)
    host_name = host_info["host_name"]
    user = host_info["user"]

    key_file = _get_private_key_file()

    subprocess.call(
        [
            "ssh",
            "-t",
            "-i",
            key_file,
            f"{user}@{host_name}",
            f"cd ~/sky_workdir; bash --login",
        ]
    )


def create_machine(
    gpus: Optional[str] = None,
    name: Optional[str] = None,
    cloud: Optional[Cloud] = None,
):
    api_client = APIClient()
    project_config = _load_project_config()

    machine_config = project_config.copy()
    if gpus:
        machine_config["resources"]["gpus"] = gpus

    if name:
        machine_config["name"] = name

    if cloud:
        machine_config["cloud"] = cloud

    if not machine_config.get("cloud", None):
        raise Exception("No cloud was provided")

    machine = api_client.create_machine(
        machine_config.get("setup", ""),
        machine_config.get("env", {}),
        machine_config.get("resources", {}),
        machine_config.get("storage", {}),
        machine_config.get("workdir", "."),
        machine_config.get("cloud", None),
        machine_config.get("docker_image", None),
        machine_config.get("name", None),
    )

    return machine


def list_machines() -> List[Machine]:
    api_client = APIClient()
    machines: List[Machine] = api_client.get_machines()

    running_machine_names = set(
        [m.name for m in machines if m.status == MachineStatus.RUNNING]
    )
    ssh_dir = os.path.expanduser("~/.komo/ssh")
    os.makedirs(ssh_dir, exist_ok=True)
    for machine_name in os.listdir(ssh_dir):
        if machine_name not in running_machine_names:
            os.remove(os.path.join(ssh_dir, machine_name))

    return machines


def terminate_machine(machine_name: str):
    api_client = APIClient()
    api_client.terminate_machine(machine_name)


def get_machine(machine_name: str) -> Machine:
    api_client = APIClient()
    machine = api_client.get_machine(machine_name)
    return machine


def _get_private_key_file():
    ssh_dir = os.path.expanduser("~/.ssh")
    os.makedirs(ssh_dir, exist_ok=True)
    key_file = os.path.join(ssh_dir, "komodo-key")
    if not os.path.isfile(key_file):
        ssh_key = _get_private_ssh_key()
        with open(key_file, "w") as f:
            f.write(ssh_key)
        os.chmod(key_file, 0o600)

    return key_file


def ssh_machine(job_id):
    api_client = APIClient()

    host_info = api_client.get_machine_host_info(job_id)
    host_name = host_info["host_name"]
    user = host_info["user"]

    key_file = _get_private_key_file()

    subprocess.call(
        [
            "ssh",
            "-t",
            "-i",
            key_file,
            f"{user}@{host_name}",
            f"cd ~/sky_workdir; bash --login",
        ]
    )


def _setup_ssh_config():
    ssh_config_file = os.path.expanduser("~/.ssh/config")
    include_entry = "Include ~/.komo/ssh/*\n"

    config = ""
    if not os.path.isfile(ssh_config_file):
        os.makedirs(os.path.expanduser("~/.ssh"), exist_ok=True)
    else:
        with open(ssh_config_file, "r") as f:
            config = f.read()

    config = include_entry + config

    with open(ssh_config_file, "w") as f:
        f.write(config)


def open_machine_in_vscode(machine_name):
    _setup_ssh_config()
    api_client = APIClient()

    code = shutil.which("code")
    if code is None:
        raise Exception(
            "Please install the VSCode CLI (https://code.visualstudio.com/docs/editor/command-line)"
        )

    host_info = api_client.get_machine_host_info(machine_name)
    key_file = _get_private_key_file()

    ssh_file = os.path.expanduser(f"~/.komo/ssh/{machine_name}")
    os.makedirs(os.path.expanduser("~/.komo/ssh"), exist_ok=True)

    with open(ssh_file, "w") as f:
        f.write(
            f"Host {machine_name}\n"
            f'\tHostname {host_info["host_name"]}\n'
            f"\tIdentityFile {key_file}\n"
            f'\tUser {host_info["user"]}\n'
        )

    subprocess.call(
        [
            "code",
            "--remote",
            f"ssh-remote+{machine_name}",
            f'/home/{host_info["user"]}/sky_workdir',
        ]
    )



import click

from komo.cli.aws.aws import aws
from komo.cli.cmd_cancel import cmd_cancel
from komo.cli.cmd_list import cmd_list
from komo.cli.cmd_login import cmd_login
from komo.cli.cmd_logs import cmd_logs
from komo.cli.cmd_register import cmd_register
from komo.cli.cmd_run import cmd_run
from komo.cli.cmd_ssh import cmd_ssh
from komo.cli.machine.machine import machine


@click.group()
@click.pass_context
def cli(ctx: click.Context):
    ctx.ensure_object(dict)


cli.add_command(cmd_run)
cli.add_command(cmd_list)
cli.add_command(cmd_logs)
cli.add_command(cmd_login)
cli.add_command(cmd_cancel)
cli.add_command(cmd_ssh)
cli.add_command(aws)
cli.add_command(machine)
cli.add_command(cmd_register)



import os
from typing import List, Optional

import click

from komo import printing
from komo.core import login


@click.command("login")
def cmd_login():
    email = click.prompt("email")
    password = click.prompt("password", hide_input=True)

    login(email, password)
    printing.success("You are now logged in!")



import click

from komo import printing
from komo.core import get_machine, ssh_machine
from komo.types import MachineStatus


@click.command("ssh")
@click.argument(
    "machine_name",
    type=str,
)
def cmd_ssh(machine_name: str):
    machine = get_machine(machine_name)
    if machine.status != MachineStatus.RUNNING:
        printing.error(f"Machine {machine_name} is not running")
        exit(1)

    ssh_machine(machine_name)



import click

from komo import printing
from komo.core import get_job, ssh_job
from komo.types import JobStatus


@click.command("ssh")
@click.argument(
    "job_id",
    type=str,
)
def cmd_ssh(job_id: str):
    job = get_job(job_id)
    if job.status != JobStatus.RUNNING:
        printing.error(f"Job {job_id} is not running")
        exit(1)

    ssh_job(job_id)



import os
from typing import List, Optional

import click

from komo import printing
from komo.core import register


@click.command("register")
def cmd_register():
    email = click.prompt("email")
    password = click.prompt("password", hide_input=True)

    register(email, password)
    printing.success("You are now registered and logged in!")



import os
import time
from typing import List, Optional

import click

from komo import printing
from komo.core import get_job, print_job_logs, run_command, run_job_file
from komo.types import Cloud, JobStatus


@click.command("run")
@click.option("--num-nodes", type=Optional[int], default=None)
@click.option("--gpus", type=str, default=None)
@click.option("--name", type=str, default=None)
@click.option("--cloud", "-c", type=str, default=None)
@click.option("--detach", "-d", is_flag=True)
@click.argument("args", nargs=-1)
def cmd_run(
    num_nodes: Optional[int],
    gpus: Optional[str],
    name: Optional[str],
    cloud: Optional[str],
    detach: bool,
    args: List[str],
):
    if cloud:
        cloud = Cloud(cloud)

    if len(args) == 1 and os.path.isfile(args[0]):
        job = run_job_file(args[0], num_nodes, gpus, name, cloud)
    else:
        job = run_command(" ".join(args), num_nodes, gpus, name, cloud)

    printing.success(f"Created job {job.name} (ID {job.id})")

    if detach:
        return

    last_message = None
    printing.info("Waiting for job to start...")
    while True:
        job = get_job(job.id)

        should_break = False
        error = False
        if job.status in [JobStatus.PENDING, JobStatus.INITIALIZING]:
            pass
        elif job.status in [
            JobStatus.RUNNING,
            JobStatus.FINISHED,
            JobStatus.SHUTTING_DOWN,
        ]:
            should_break = True
        else:
            printing.error(f"Job status is {job.status.name}")
            should_break = True
            error = True

        if job.status_message and job.status_message != last_message:
            if error:
                printing.error(job.status_message)
            else:
                printing.info(job.status_message)

            last_message = job.status_message

        if should_break:
            break

        time.sleep(2)

    if job.status in [JobStatus.RUNNING, JobStatus.FINISHED, JobStatus.SHUTTING_DOWN]:
        printing.success(f"Job successfully started")

        print_job_logs(job.id, True)
